%% Project:   High Angular Resolution Diffusion Imaging Tools
%  Function to compute the ODF corresponding to the Diffusion Orientation Transform (DOT) method 
%  using an expansion in terms of real spherical harmonic basis.
%  The ODF is a contour of the propagator, evaluated at a given displacement length.
%  It is assumed that the measurement scheme is a single spherical shell.  
%                                                                                                
%   Language:  MATLAB(R)
%   Author:  Erick Canales-Rodríguez, Lester Melie-García, Yasser Iturria-Medina, Yasser Alemán-Gómez
%   Date: 2013, Version: 1.2
% 
%  See also test_DOT_example, mchgm

function ODF = ReconstructionDOT(A, Lmax, basisV, K, Ro, Dt, Matrix)

long = 0:2:Lmax;
for i = 1:length(long)
    DTL = Dt.^( (long(i)+ 3)/2 );
    a1 = long(i)/2 + 3/2; a2 = long(i) + 3/2; a3 = -(Ro.^2)./(4*Dt);
    F1x = mchgm(a1, a2, a3); % mchgm is the hypergeometric1F1 function
    ILu_i = (Ro.^long(i)).*F1x./( pi^(3/2)*DTL.*( 2^(long(i)+3) ) ); 
    sDOT(:,i) = A*ILu_i(:);
end

sDOT = sum(sDOT.*Matrix,2);
ss = sDOT.*K;
ODF = basisV*ss;

% Some options to avoid negative sidelobes from the reconstruction:
% #1)
%ODF(ODF<0)=0;
%
% #2)
%T = abs(min(ODF));
%ODF = wthresh(ODF,'s',T); % soft-thresholding
%
% #3)
%ODF = abs(ODF);
return
